package jalview.datamodel;

import jalview.schemes.*;
import jalview.analysis.*;
import jalview.datamodel.*;
import java.util.Vector;

public class SequenceGroup
{
  String groupName;
  boolean isSelected;
  boolean displayBoxes;
  boolean displayText;
  boolean colourText;
  boolean display;
  Conservation conserve;
  Vector aaFrequency;
  boolean aaFrequencyValid = false;
  public Vector sequences = new Vector();
  int width = -1;
  public ColourSchemeI cs;
  int startRes = 20;
  int endRes = 35;

  public SequenceGroup()
  {
    groupName = "Group";
    this.isSelected = false;
    this.displayBoxes = true;
    this.displayText = true;
    this.colourText = false;
    this.display = true;
    cs = null;
  }

  public SequenceGroup(String groupName, ColourSchemeI scheme,
                       boolean isSelected,
                       boolean displayBoxes, boolean displayText,
                       boolean colourText,
                       boolean display)
  {

    this.groupName = groupName;
    this.isSelected = isSelected;
    this.displayBoxes = displayBoxes;
    this.displayText = displayText;
    this.colourText = colourText;
    this.display = display;
    this.cs = scheme;
  }

  public String getName()
  {
    return groupName;
  }

  public void setName(String name)
  {
    groupName = name;
  }

  public Conservation getConservation()
  {
    return conserve;
  }

  public void setConservation(Conservation c)
  {
    conserve = c;
  }

  public void addSequence(SequenceI s)
  {
    if(!sequences.contains(s))
      sequences.addElement(s);
  }

  public void deleteSequence(SequenceI s)
  {
    sequences.removeElement(s);
  }

  public void setColourText(boolean state)
  {
    colourText = state;
  }

  public boolean getColourText()
  {
    return colourText;
  }

  public void setDisplayText(boolean state)
  {
    displayText = state;
  }

  public boolean getDisplayText()
  {
    return displayText;
  }

  public void setDisplayBoxes(boolean state)
  {
    displayBoxes = state;
  }

  public boolean getDisplayBoxes()
  {
    return displayBoxes;
  }

  public int getStartRes()
  {
    return startRes;
  }

  public int getEndRes()
  {
    return endRes;
  }

  public void setStartRes(int i)
  {
    startRes = i;
  }

  public void setEndRes(int i)
  {
    endRes = i;
  }

  public int getSize()
  {
    return sequences.size();
  }

  public SequenceI getSequenceAt(int i)
  {
    return (SequenceI) sequences.elementAt(i);
  }

  public Vector getAAFrequency()
  {
    if (aaFrequency == null || aaFrequencyValid == false)
    {
      aaFrequency = AAFrequency.calculate(sequences, 0, getWidth());
      aaFrequencyValid = true;
    }
    return aaFrequency;
  }

  public int getWidth()
  {
    System.out.println("it is called");
    // MC This needs to get reset when characters are inserted and deleted
    for (int i = 0; i < sequences.size(); i++)
    {
      SequenceI seq = (SequenceI) sequences.elementAt(i);
      if (seq.getLength() > width)
        width = seq.getLength();
    }
    return width;
  }
}
