package jalview.gui;

import jalview.jbgui.GAlignmentPanel;
import jalview.schemes.*;
import jalview.analysis.*;
import jalview.datamodel.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;

public class AlignmentPanel extends GAlignmentPanel implements AdjustmentListener, Printable
{

  AlignViewport     av;
  OverviewPanel overviewPanel;
  SeqPanel   seqPanel;
  IdPanel    idPanel;
  public AlignFrame alignFrame;
  ScalePanel scalePanel;
  ScorePanel scorePanel;

  public AlignmentPanel(AlignFrame af, AlignViewport av)
  {
    alignFrame = af;
    this.av         = av;
    seqPanel        = new SeqPanel  (av, this);
    idPanel         = new IdPanel   (av, this);
    scalePanel = new ScalePanel(av);
    scorePanel = new ScorePanel(av);

    idPanelHolder.add(idPanel, BorderLayout.CENTER);
    idPanel.addNotify();
    scalePanelHolder.add(scalePanel, BorderLayout.CENTER);
    scorePanelHolder.add(scorePanel, BorderLayout.CENTER);
    seqPanelHolder.add(seqPanel, BorderLayout.CENTER);
    setScrollValues(0,0);

    hscroll.addAdjustmentListener(this);
    vscroll.addAdjustmentListener(this);

    addComponentListener(new ComponentAdapter()
   {
       public void componentResized(ComponentEvent evt)
     {
          RefreshPanels();
     }
   });
  }

 public void highlightSearchResults(int [] results)
 {
   seqPanel.seqCanvas.highlightSearchResults( results );

   // do we need to scroll the panel?
   if(results!=null && (av.getStartSeq()>results[0]
                        || av.getEndSeq()<results[0]
                        || av.getStartRes()>results[1]
                        || av.getEndRes()<results[2]))
       setScrollValues(results[1], results[0]);


 }


 public OverviewPanel getOverviewPanel()
 {
   return overviewPanel;
 }

 public void setOverviewPanel(OverviewPanel op)
 {
   overviewPanel = op;
 }


  public void setGraphPanelVisible(boolean b)
  {
    scorePanelHolder.setVisible(b);
    idSpaceFillerPanel.setVisible(b);
    RefreshPanels();
  }

  public void setWrapAlignment(boolean wrap)
  {
      scalePanelHolder.setVisible(!wrap);
      scorePanelHolder.setVisible(!wrap);

      hscroll.setVisible(!wrap);
      idSpaceFillerPanel.setVisible(!wrap);
      idSpaceFillerPanel1.setVisible(!wrap);
  }


  public void setColourScheme()
  {
    ColourSchemeI cs = av.getGlobalColourScheme();

    if(av.getConservationSelected())
    {

      Alignment al = (Alignment)av.getAlignment();
      Conservation c = new Conservation("All", al.cons,
                            ResidueProperties.propHash, 3, al.getSequences(), 0,
                            al.getWidth() );

       c.calculate();
       c.verdict(false, 100);
       ConservationColourScheme ccs = new ConservationColourScheme(c, cs);

       av.setGlobalColourScheme( ccs );

    }
    seqPanel.seqCanvas.paintFlag = true;
    repaint();
  }
  /*
    seqCanvas.paintFlag = true;
    if (av.getSelection().size() == 0)
    {
      seqCanvas.globalColorScheme = cs;
      for (int i = 0; i < av.alignment.getGroups().size();i++)
      {
        SequenceGroup sg = (SequenceGroup)av.alignment.getGroups().elementAt(i);
        sg.cs = cs;
        if(!showConservation
            && !(sg.cs instanceof Blosum62ColourScheme)
            && sg.cs instanceof ConservationColourScheme)
         {
           // remove ConservationColouring from existing Conservation group
           ConservationColourScheme ccs = (ConservationColourScheme)sg.cs;
           sg.cs = ccs.cs;
         }
         else if(showConservation && !(sg.cs instanceof ConservationColourScheme))
         {
           // add ConservationColouring to new group
           Conservation c = sg.getConservation();
           Alignment al = (Alignment) av.getAlignment();
           c = new Conservation("All", al.cons,
                                ResidueProperties.propHash, 3, sg.sequences, 0,
                                al.getWidth() );
           c.calculate();
           c.verdict(false, 100);
           sg.setConservation(c);
           sg.cs = new ConservationColourScheme(sg);
         }
      }
    }
    else
    {
      SequenceGroup sg = av.alignment.findGroup((Sequence)av.sel.sequenceAt(0));

      if( isNewSelection(sg) )
      {
        sg = av.getAlignment().addGroup();
        for (int i=0; i < av.getSelection().size(); i++)
        {
          av.alignment.removeFromGroup(av.alignment.findGroup( (Sequence) av.
              sel.sequenceAt(i)),
                                       (Sequence) av.sel.sequenceAt(i));
          av.alignment.addToGroup(sg, (Sequence) av.sel.sequenceAt(i));
        }
      }


      sg.cs = cs;

      // Selection is made, we only want to change the conservationColour for selected group
      if(  showConservation
           && !(sg.cs instanceof ConservationColourScheme)
           && !(sg.cs instanceof Blosum62ColourScheme))
      {
        Conservation c = sg.getConservation();
        Alignment al = (Alignment) av.getAlignment();

        c = new Conservation("All", al.cons,
                             ResidueProperties.propHash, 3, sg.sequences, 0,
                             al.getWidth() );
        c.calculate();
        c.verdict(false, 100);
        sg.setConservation(c);
        sg.cs = new ConservationColourScheme(sg);
    }
    else  if( !showConservation && sg.cs instanceof ConservationColourScheme)
    {
      ConservationColourScheme ccs = (ConservationColourScheme)sg.cs;
      sg.cs = ccs.cs;
    }
   }
   repaint();
  }

  boolean isNewSelection(SequenceGroup sg)
  {
    if(sg==null)
      return true;

    if(sg.getSize()!=av.getSelection().size())
      return true;

    for(int i=0; i<sg.getSize(); i++)
     if( !av.getSelection().contains( sg.getSequenceAt(i) ))
       return true;

    return false;
  }


  public ColourSchemeI getColourScheme()
  {
    if(av.getSelection().size()>0)
       return av.alignment.findGroup((Sequence)av.sel.sequenceAt(0)).cs;
    else
      return seqCanvas.globalColorScheme;
  }
*/

/*  public void setColourScheme(jalview.schemes.ColourSchemeI cs, boolean conservation)
  {
    seqPanel.setColourScheme(cs, conservation);

    av.setGlobalColourScheme(cs);

    if(overviewPanel!=null)
      overviewPanel.updateOverviewImage();
  }

   void updateResidueView()
   {
     if (viewport.getSelection().size() == 0)
      {
        for (int i = 0; i < viewport.alignment.getGroups().size(); i++)
        {
          SequenceGroup sg = (SequenceGroup) viewport.alignment.getGroups().elementAt(i);
          sg.setDisplayBoxes( viewBoxesMenuItem.isSelected() );
          sg.setDisplayText( viewTextMenuItem.isSelected() );
          sg.setColourText( colourTextMenuItem.isSelected() );
        }
      }
      else
      {
        SequenceGroup sg = viewport.alignment.findGroup( (Sequence) viewport.sel.sequenceAt(0));
        if (alignPanel.seqPanel.isNewSelection(sg))
        {
          sg = viewport.getAlignment().addGroup();
          for (int i = 0; i < viewport.getSelection().size(); i++)
          {
            viewport.alignment.removeFromGroup(viewport.alignment.findGroup( (
                Sequence) viewport.sel.sequenceAt(i)),
                                               (Sequence) viewport.sel.sequenceAt(
                i));
            viewport.alignment.addToGroup(sg,
                                          (Sequence) viewport.sel.sequenceAt(i));
          }
        }

        sg.setDisplayBoxes(viewBoxesMenuItem.isSelected());
        sg.setDisplayText(viewTextMenuItem.isSelected());
        sg.setColourText(colourTextMenuItem.isSelected());
      }
     alignPanel.RefreshPanels();
   }*/



  public void RefreshPanels()
  {
    invalidate();
    seqPanel.seqCanvas.paintFlag = true;
    idPanel.idCanvas.paintFlag = true;
    idPanelHolder.setPreferredSize( idPanel.idCanvas.getPreferredSize() );
    setScrollValues(av.getStartRes(), av.getStartSeq());
    av.getConsensus(true);
    validate();
    repaint();
    if(overviewPanel!=null)
      overviewPanel.updateOverviewImage();

  }

  int hextent = 0;
  int vextent = 0;

  public void setScrollValues(int x, int y)
  {
    hextent = seqPanel.seqCanvas.getWidth()/av.getCharWidth();
    vextent = seqPanel.seqCanvas.getHeight()/av.getCharHeight();

    if(hextent+x  >  av.getAlignment().getWidth())
      x =  av.getAlignment().getWidth()- hextent;

    if(vextent+y > av.getAlignment().getHeight())
      y = av.getAlignment().getHeight() - vextent;

    if(y<0)
      y = 0;

    if(x<0)
      x=0;

    hscroll.setValues(x,hextent,0,av.getAlignment().getWidth());
    vscroll.setValues(y,vextent,0,av.getAlignment().getHeight() );
    repaint();

  }


  public void adjustmentValueChanged(AdjustmentEvent evt)
  {

    if (evt.getSource() == hscroll)
    {
      int x = hscroll.getValue();
      av.setStartRes(x);
      av.setEndRes(x + seqPanel.seqCanvas.getWidth()/av.getCharWidth()-1);
    }

    if (evt.getSource() == vscroll)
    {
        int offy = vscroll.getValue();
        av.setStartSeq(offy);
        av.setEndSeq(offy + seqPanel.seqCanvas.getHeight()/av.getCharHeight());
    }


    if(overviewPanel!=null)
      overviewPanel.setBoxPosition();

    seqPanel.seqCanvas.paintFlag=true;
    repaint();
  }


  public int print(Graphics pg, PageFormat pf, int pi) throws PrinterException
  {

    pg.translate((int)pf.getImageableX(), (int)pf.getImageableY());

    int pwidth = (int)pf.getImageableWidth();
    int pheight = (int)pf.getImageableHeight();
    int idWidth = (int)idPanel.idCanvas.getLabelWidth().getWidth();

    pg.setColor(Color.white);
    pg.fillRect(0,0,pwidth, pheight);
    pg.setFont( av.getFont() );

    ////////////////////////////////////
    /// How many sequences and residues can we fit on a printable page?
    int totalRes = (pwidth - idWidth)/av.getCharWidth();
    int totalSeq = (int)((pheight - 30)/av.getCharHeight())-1;
    int pagesWide = av.getAlignment().getWidth() / totalRes +1;
    int pagesHigh = av.getAlignment().getHeight() / totalSeq +1;

    if (pi >= pagesWide*pagesHigh)
     return Printable.NO_SUCH_PAGE;

    /////////////////////////////
    /// Only print these sequences and residues on this page
    int startRes, endRes, startSeq, endSeq;
    startRes = (pi % pagesWide) * totalRes;
    endRes = startRes + totalRes-1;
    if(endRes>av.getAlignment().getWidth())
      endRes = av.getAlignment().getWidth();

     startSeq = (pi / pagesWide) * totalSeq;
     endSeq = startSeq + totalSeq;
     if(endSeq > av.getAlignment().getHeight())
       endSeq = av.getAlignment().getHeight();


    ////////////////
    //draw Scale
    pg.translate(idWidth,0);
    scalePanel.scaleCanvas.drawScale(pg, startRes, endRes, pwidth-idWidth);

    pg.translate(-idWidth, 30);
    ////////////////
    // Draw the ids
    Color currentColor=null;
    Color currentTextColor=null;
    for(int i=startSeq; i<endSeq; i++)
    {
      if (av.getSelection().contains(av.getAlignment().getSequenceAt(i)))
      {
        currentColor = Color.gray;
        currentTextColor = Color.black;
      }
      else
      {
        currentColor = av.getAlignment().getSequenceAt(i).getColor();
        currentTextColor = Color.black;
      }

      pg.setColor(currentColor);
      pg.fillRect(0,  jalview.analysis.AlignmentUtil.getPixelHeight(startSeq, i, av.getCharHeight()),
                              idWidth,
                              av.getCharHeight());

      pg.setColor(currentTextColor);

      String string = av.getAlignment().getSequenceAt(i).getDisplayId();
      pg.drawString(string, 0,  jalview.analysis.AlignmentUtil.getPixelHeight
                    (startSeq, i, av.getCharHeight()) + av.getCharHeight() - (av.getCharHeight() / 5));
    }

    // draw main sequence panel
    pg.translate(idWidth,0);
    pg.setClip(0,0,pwidth-idWidth, pheight);
    seqPanel.seqCanvas.drawPanel(pg,startRes,endRes,startSeq,endSeq,startRes,startSeq,0);

    return Printable.PAGE_EXISTS;
  }

}



